@@ -6,9 +6,12 @@
 
 #include <common.h>
 #include <debug_uart.h>
+#include <env.h>
+#include <init.h>
 #include <led.h>
 #include <asm/arch/at91_common.h>
 #include <asm/arch/clk.h>
+#include <asm/global_data.h>
 
 DECLARE_GLOBAL_DATA_PTR;
 
@@ -21,9 +24,6 @@ int board_late_init(void)
 {
 	at91_prepare_cpu_var();
 
-	if (IS_ENABLED(CONFIG_LED))
-		led_default_state();
-
 	return 0;
 }
 
