@@ -6,10 +6,12 @@
  */
 
 #include <common.h>
+#include <display_options.h>
 #include <dm.h>
 #include <eeprom.h>
 #include <init.h>
 #include <log.h>
+#include <asm/global_data.h>
 #include <dm/device-internal.h>
 #include <ahci.h>
 #include <env.h>
