@@ -10,6 +10,7 @@
 #include <log.h>
 #include <spi.h>
 #include <lcd.h>
+#include <asm/global_data.h>
 #include <asm/io.h>
 #include <asm/gpio.h>
 #include <asm/arch/adc.h>
@@ -32,10 +33,12 @@ DECLARE_GLOBAL_DATA_PTR;
 unsigned int board_rev;
 static int init_pmic_lcd(void);
 
+#ifdef CONFIG_REVISION_TAG
 u32 get_board_rev(void)
 {
 	return board_rev;
 }
+#endif
 
 int exynos_power_init(void)
 {
@@ -264,98 +267,6 @@ static int init_pmic_lcd(void)
 	return 0;
 }
 
-void exynos_cfg_lcd_gpio(void)
-{
-	unsigned int i, f3_end = 4;
-
-	for (i = 0; i < 8; i++) {
-		/* set GPF0,1,2[0:7] for RGB Interface and Data lines (32bit) */
-		gpio_cfg_pin(EXYNOS4_GPIO_F00 + i, S5P_GPIO_FUNC(2));
-		gpio_cfg_pin(EXYNOS4_GPIO_F10 + i, S5P_GPIO_FUNC(2));
-		gpio_cfg_pin(EXYNOS4_GPIO_F20 + i, S5P_GPIO_FUNC(2));
-		/* pull-up/down disable */
-		gpio_set_pull(EXYNOS4_GPIO_F00 + i, S5P_GPIO_PULL_NONE);
-		gpio_set_pull(EXYNOS4_GPIO_F10 + i, S5P_GPIO_PULL_NONE);
-		gpio_set_pull(EXYNOS4_GPIO_F20 + i, S5P_GPIO_PULL_NONE);
-
-		/* drive strength to max (24bit) */
-		gpio_set_drv(EXYNOS4_GPIO_F00 + i, S5P_GPIO_DRV_4X);
-		gpio_set_rate(EXYNOS4_GPIO_F00 + i, S5P_GPIO_DRV_SLOW);
-		gpio_set_drv(EXYNOS4_GPIO_F10 + i, S5P_GPIO_DRV_4X);
-		gpio_set_rate(EXYNOS4_GPIO_F10 + i, S5P_GPIO_DRV_SLOW);
-		gpio_set_drv(EXYNOS4_GPIO_F20 + i, S5P_GPIO_DRV_4X);
-		gpio_set_rate(EXYNOS4_GPIO_F00 + i, S5P_GPIO_DRV_SLOW);
-	}
-
-	for (i = EXYNOS4_GPIO_F30; i < (EXYNOS4_GPIO_F30 + f3_end); i++) {
-		/* set GPF3[0:3] for RGB Interface and Data lines (32bit) */
-		gpio_cfg_pin(i, S5P_GPIO_FUNC(2));
-		/* pull-up/down disable */
-		gpio_set_pull(i, S5P_GPIO_PULL_NONE);
-		/* drive strength to max (24bit) */
-		gpio_set_drv(i, S5P_GPIO_DRV_4X);
-		gpio_set_rate(i, S5P_GPIO_DRV_SLOW);
-	}
-
-	/* gpio pad configuration for LCD reset. */
-	gpio_request(EXYNOS4_GPIO_Y45, "lcd_reset");
-	gpio_cfg_pin(EXYNOS4_GPIO_Y45, S5P_GPIO_OUTPUT);
-}
-
-int mipi_power(void)
-{
-	return 0;
-}
-
-void exynos_reset_lcd(void)
-{
-	gpio_set_value(EXYNOS4_GPIO_Y45, 1);
-	udelay(10000);
-	gpio_set_value(EXYNOS4_GPIO_Y45, 0);
-	udelay(10000);
-	gpio_set_value(EXYNOS4_GPIO_Y45, 1);
-	udelay(100);
-}
-
-void exynos_lcd_power_on(void)
-{
-	struct udevice *dev;
-	int ret;
-	u8 reg;
-
-	ret = pmic_get("max8998-pmic", &dev);
-	if (ret) {
-		puts("Failed to get MAX8998!\n");
-		return;
-	}
-
-	reg = pmic_reg_read(dev, MAX8998_REG_ONOFF3);
-	reg |= MAX8998_LDO17;
-	ret = pmic_reg_write(dev, MAX8998_REG_ONOFF3, reg);
-	if (ret) {
-		puts("MAX8998 LDO setting error\n");
-		return;
-	}
-
-	reg = pmic_reg_read(dev, MAX8998_REG_ONOFF2);
-	reg |= MAX8998_LDO7;
-	ret = pmic_reg_write(dev, MAX8998_REG_ONOFF2, reg);
-	if (ret) {
-		puts("MAX8998 LDO setting error\n");
-		return;
-	}
-}
-
-void exynos_cfg_ldo(void)
-{
-	ld9040_cfg_ldo();
-}
-
-void exynos_enable_ldo(unsigned int onoff)
-{
-	ld9040_enable_ldo(onoff);
-}
-
 int exynos_init(void)
 {
 	gd->bd->bi_arch_number = MACH_TYPE_UNIVERSAL_C210;
@@ -387,18 +298,3 @@ int exynos_init(void)
 
 	return 0;
 }
-
-#ifdef CONFIG_LCD
-void exynos_lcd_misc_init(vidinfo_t *vid)
-{
-#ifdef CONFIG_TIZEN
-	get_tizen_logo_info(vid);
-#endif
-
-	/* for LD9040. */
-	vid->pclk_name = 1;	/* MPLL */
-	vid->sclk_div = 1;
-
-	env_set("lcdinfo", "lcd=ld9040");
-}
-#endif
