@@ -4,33 +4,19 @@
  */
 
 #include <common.h>
+#include <asm/global_data.h>
 #include "tdx-cfg-block.h"
 #include "tdx-eeprom.h"
 
 #include <command.h>
 #include <asm/cache.h>
 
-#if defined(CONFIG_TARGET_APALIS_IMX6) || \
-	defined(CONFIG_TARGET_APALIS_IMX8) || \
-	defined(CONFIG_TARGET_APALIS_IMX8X) || \
-	defined(CONFIG_TARGET_COLIBRI_IMX6) || \
-	defined(CONFIG_TARGET_COLIBRI_IMX8X) || \
-	defined(CONFIG_TARGET_VERDIN_IMX8MM) || \
-	defined(CONFIG_TARGET_VERDIN_IMX8MN) || \
-	defined(CONFIG_TARGET_VERDIN_IMX8MP)
-#include <asm/arch/sys_proto.h>
-#else
-#define is_cpu_type(cpu) (0)
-#endif
-#if defined(CONFIG_CPU_PXA27X)
-#include <asm/arch-pxa/pxa.h>
-#else
-#define cpu_is_pxa27x(cpu) (0)
-#endif
 #include <cli.h>
 #include <console.h>
 #include <env.h>
+#ifdef CONFIG_TDX_CFG_BLOCK_IS_IN_NOR
 #include <flash.h>
+#endif
 #include <malloc.h>
 #include <mmc.h>
 #include <nand.h>
@@ -81,69 +67,78 @@ bool valid_cfgblock_carrier;
 struct toradex_hw tdx_car_hw_tag;
 #endif
 
-const char * const toradex_modules[] = {
-	 [0] = "UNKNOWN MODULE",
-	 [1] = "Colibri PXA270 312MHz",
-	 [2] = "Colibri PXA270 520MHz",
-	 [3] = "Colibri PXA320 806MHz",
-	 [4] = "Colibri PXA300 208MHz",
-	 [5] = "Colibri PXA310 624MHz",
-	 [6] = "Colibri PXA320 806MHz IT",
-	 [7] = "Colibri PXA300 208MHz XT",
-	 [8] = "Colibri PXA270 312MHz",
-	 [9] = "Colibri PXA270 520MHz",
-	[10] = "Colibri VF50 128MB", /* not currently on sale */
-	[11] = "Colibri VF61 256MB",
-	[12] = "Colibri VF61 256MB IT",
-	[13] = "Colibri VF50 128MB IT",
-	[14] = "Colibri iMX6 Solo 256MB",
-	[15] = "Colibri iMX6 DualLite 512MB",
-	[16] = "Colibri iMX6 Solo 256MB IT",
-	[17] = "Colibri iMX6 DualLite 512MB IT",
-	[18] = "UNKNOWN MODULE",
-	[19] = "UNKNOWN MODULE",
-	[20] = "Colibri T20 256MB",
-	[21] = "Colibri T20 512MB",
-	[22] = "Colibri T20 512MB IT",
-	[23] = "Colibri T30 1GB",
-	[24] = "Colibri T20 256MB IT",
-	[25] = "Apalis T30 2GB",
-	[26] = "Apalis T30 1GB",
-	[27] = "Apalis iMX6 Quad 1GB",
-	[28] = "Apalis iMX6 Quad 2GB IT",
-	[29] = "Apalis iMX6 Dual 512MB",
-	[30] = "Colibri T30 1GB IT",
-	[31] = "Apalis T30 1GB IT",
-	[32] = "Colibri iMX7 Solo 256MB",
-	[33] = "Colibri iMX7 Dual 512MB",
-	[34] = "Apalis TK1 2GB",
-	[35] = "Apalis iMX6 Dual 1GB IT",
-	[36] = "Colibri iMX6ULL 256MB",
-	[37] = "Apalis iMX8 QuadMax 4GB Wi-Fi / BT IT",
-	[38] = "Colibri iMX8 QuadXPlus 2GB Wi-Fi / BT IT",
-	[39] = "Colibri iMX7 Dual 1GB (eMMC)",
-	[40] = "Colibri iMX6ULL 512MB Wi-Fi / BT IT",
-	[41] = "Colibri iMX7 Dual 512MB EPDC",
-	[42] = "Apalis TK1 4GB",
-	[43] = "Colibri T20 512MB IT SETEK",
-	[44] = "Colibri iMX6ULL 512MB IT",
-	[45] = "Colibri iMX6ULL 512MB Wi-Fi / Bluetooth",
-	[46] = "Apalis iMX8 QuadXPlus 2GB Wi-Fi / BT IT",
-	[47] = "Apalis iMX8 QuadMax 4GB IT",
-	[48] = "Apalis iMX8 QuadPlus 2GB Wi-Fi / BT",
-	[49] = "Apalis iMX8 QuadPlus 2GB",
-	[50] = "Colibri iMX8 QuadXPlus 2GB IT",
-	[51] = "Colibri iMX8 DualX 1GB Wi-Fi / Bluetooth",
-	[52] = "Colibri iMX8 DualX 1GB",
-	[53] = "Apalis iMX8 QuadXPlus 2GB ECC IT",
-	[54] = "Apalis iMX8 DualXPlus 1GB",
-	[55] = "Verdin iMX8M Mini Quad 2GB Wi-Fi / BT IT",
-	[56] = "Verdin iMX8M Nano Quad 1GB Wi-Fi / BT", /* not currently on sale */
-	[57] = "Verdin iMX8M Mini DualLite 1GB",
-	[58] = "Verdin iMX8M Plus Quad 4GB Wi-Fi / BT IT",
-	[59] = "Verdin iMX8M Mini Quad 2GB IT",
-	[60] = "Verdin iMX8M Mini DualLite 1GB WB IT",
-	[61] = "Verdin iMX8M Plus Quad 2GB",
+#define TARGET_IS_ENABLED(x) IS_ENABLED(CONFIG_TARGET_ ## x)
+
+const struct toradex_som toradex_modules[] = {
+	 [0] = { "UNKNOWN MODULE",                       0                                  },
+	 [1] = { "Colibri PXA270 312MHz",                0                                  },
+	 [2] = { "Colibri PXA270 520MHz",                0                                  },
+	 [3] = { "Colibri PXA320 806MHz",                0                                  },
+	 [4] = { "Colibri PXA300 208MHz",                0                                  },
+	 [5] = { "Colibri PXA310 624MHz",                0                                  },
+	 [6] = { "Colibri PXA320IT 806MHz",              0                                  },
+	 [7] = { "Colibri PXA300 208MHz XT",             0                                  },
+	 [8] = { "Colibri PXA270 312MHz",                0                                  },
+	 [9] = { "Colibri PXA270 520MHz",                0                                  },
+	[10] = { "Colibri VF50 128MB",                   TARGET_IS_ENABLED(COLIBRI_VF)      },
+	[11] = { "Colibri VF61 256MB",                   TARGET_IS_ENABLED(COLIBRI_VF)      },
+	[12] = { "Colibri VF61 256MB IT",                TARGET_IS_ENABLED(COLIBRI_VF)      },
+	[13] = { "Colibri VF50 128MB IT",                TARGET_IS_ENABLED(COLIBRI_VF)      },
+	[14] = { "Colibri iMX6S 256MB",                  TARGET_IS_ENABLED(COLIBRI_IMX6)    },
+	[15] = { "Colibri iMX6DL 512MB",                 TARGET_IS_ENABLED(COLIBRI_IMX6)    },
+	[16] = { "Colibri iMX6S 256MB IT",               TARGET_IS_ENABLED(COLIBRI_IMX6)    },
+	[17] = { "Colibri iMX6DL 512MB IT",              TARGET_IS_ENABLED(COLIBRI_IMX6)    },
+	[18] = { "UNKNOWN MODULE",                       0                                  },
+	[19] = { "UNKNOWN MODULE",                       0                                  },
+	[20] = { "Colibri T20 256MB",                    TARGET_IS_ENABLED(COLIBRI_T20)     },
+	[21] = { "Colibri T20 512MB",                    TARGET_IS_ENABLED(COLIBRI_T20)     },
+	[22] = { "Colibri T20 512MB IT",                 TARGET_IS_ENABLED(COLIBRI_T20)     },
+	[23] = { "Colibri T30 1GB",                      TARGET_IS_ENABLED(COLIBRI_T30)     },
+	[24] = { "Colibri T20 256MB IT",                 TARGET_IS_ENABLED(COLIBRI_T20)     },
+	[25] = { "Apalis T30 2GB",                       TARGET_IS_ENABLED(APALIS_T30)      },
+	[26] = { "Apalis T30 1GB",                       TARGET_IS_ENABLED(APALIS_T30)      },
+	[27] = { "Apalis iMX6Q 1GB",                     TARGET_IS_ENABLED(APALIS_IMX6)     },
+	[28] = { "Apalis iMX6Q 2GB IT",                  TARGET_IS_ENABLED(APALIS_IMX6)     },
+	[29] = { "Apalis iMX6D 512MB",                   TARGET_IS_ENABLED(APALIS_IMX6)     },
+	[30] = { "Colibri T30 1GB IT",                   TARGET_IS_ENABLED(COLIBRI_T30)     },
+	[31] = { "Apalis T30 1GB IT",                    TARGET_IS_ENABLED(APALIS_T30)      },
+	[32] = { "Colibri iMX7S 256MB",                  TARGET_IS_ENABLED(COLIBRI_IMX7)    },
+	[33] = { "Colibri iMX7D 512MB",                  TARGET_IS_ENABLED(COLIBRI_IMX7)    },
+	[34] = { "Apalis TK1 2GB",                       TARGET_IS_ENABLED(APALIS_TK1)      },
+	[35] = { "Apalis iMX6D 1GB IT",                  TARGET_IS_ENABLED(APALIS_IMX6)     },
+	[36] = { "Colibri iMX6ULL 256MB",                TARGET_IS_ENABLED(COLIBRI_IMX6ULL) },
+	[37] = { "Apalis iMX8QM 4GB WB IT",              TARGET_IS_ENABLED(APALIS_IMX8)     },
+	[38] = { "Colibri iMX8QXP 2GB WB IT",            TARGET_IS_ENABLED(COLIBRI_IMX8X)   },
+	[39] = { "Colibri iMX7D 1GB",                    TARGET_IS_ENABLED(COLIBRI_IMX7)    },
+	[40] = { "Colibri iMX6ULL 512MB WB IT",          TARGET_IS_ENABLED(COLIBRI_IMX6ULL) },
+	[41] = { "Colibri iMX7D 512MB EPDC",             TARGET_IS_ENABLED(COLIBRI_IMX7)    },
+	[42] = { "Apalis TK1 4GB",                       TARGET_IS_ENABLED(APALIS_TK1)      },
+	[43] = { "Colibri T20 512MB IT SETEK",           TARGET_IS_ENABLED(COLIBRI_T20)     },
+	[44] = { "Colibri iMX6ULL 512MB IT",             TARGET_IS_ENABLED(COLIBRI_IMX6ULL) },
+	[45] = { "Colibri iMX6ULL 512MB WB",             TARGET_IS_ENABLED(COLIBRI_IMX6ULL) },
+	[46] = { "Apalis iMX8QXP 2GB WB IT",             0                                  },
+	[47] = { "Apalis iMX8QM 4GB IT",                 TARGET_IS_ENABLED(APALIS_IMX8)     },
+	[48] = { "Apalis iMX8QP 2GB WB",                 TARGET_IS_ENABLED(APALIS_IMX8)     },
+	[49] = { "Apalis iMX8QP 2GB",                    TARGET_IS_ENABLED(APALIS_IMX8)     },
+	[50] = { "Colibri iMX8QXP 2GB IT",               TARGET_IS_ENABLED(COLIBRI_IMX8X)   },
+	[51] = { "Colibri iMX8DX 1GB WB",                TARGET_IS_ENABLED(COLIBRI_IMX8X)   },
+	[52] = { "Colibri iMX8DX 1GB",                   TARGET_IS_ENABLED(COLIBRI_IMX8X)   },
+	[53] = { "Apalis iMX8QXP 2GB ECC IT",            0                                  },
+	[54] = { "Apalis iMX8DXP 1GB",                   TARGET_IS_ENABLED(APALIS_IMX8)     },
+	[55] = { "Verdin iMX8M Mini Quad 2GB WB IT",     TARGET_IS_ENABLED(VERDIN_IMX8MM)   },
+	[56] = { "Verdin iMX8M Nano Quad 1GB WB",        0                                  },
+	[57] = { "Verdin iMX8M Mini DualLite 1GB",       TARGET_IS_ENABLED(VERDIN_IMX8MM)   },
+	[58] = { "Verdin iMX8M Plus Quad 4GB WB IT",     TARGET_IS_ENABLED(VERDIN_IMX8MP)   },
+	[59] = { "Verdin iMX8M Mini Quad 2GB IT",        TARGET_IS_ENABLED(VERDIN_IMX8MM)   },
+	[60] = { "Verdin iMX8M Mini DualLite 1GB WB IT", TARGET_IS_ENABLED(VERDIN_IMX8MM)   },
+	[61] = { "Verdin iMX8M Plus Quad 2GB",           TARGET_IS_ENABLED(VERDIN_IMX8MP)   },
+	[62] = { "Colibri iMX6ULL 1GB IT",               TARGET_IS_ENABLED(COLIBRI_IMX6ULL) },
+	[63] = { "Verdin iMX8M Plus Quad 4GB IT",        TARGET_IS_ENABLED(VERDIN_IMX8MP)   },
+	[64] = { "Verdin iMX8M Plus Quad 2GB WB IT",     TARGET_IS_ENABLED(VERDIN_IMX8MP)   },
+	[65] = { "Verdin iMX8M Plus QuadLite 1GB IT",    TARGET_IS_ENABLED(VERDIN_IMX8MP)   },
+	[66] = { "Verdin iMX8M Plus Quad 8GB WB",        TARGET_IS_ENABLED(VERDIN_IMX8MP)   },
+	[67] = { "Apalis iMX8QM 8GB WB IT",              TARGET_IS_ENABLED(APALIS_IMX8)     },
+	[68] = { "Verdin iMX8M Mini Quad 2GB WB IT",     TARGET_IS_ENABLED(VERDIN_IMX8MM)   },
 };
 
 const char * const toradex_carrier_boards[] = {
@@ -158,6 +153,42 @@ const char * const toradex_display_adapters[] = {
 	[159] = "Verdin DSI to LVDS Adapter",
 };
 
+const u32 toradex_ouis[] = {
+	[0] = 0x00142dUL,
+	[1] = 0x8c06cbUL,
+};
+
+static u32 get_serial_from_mac(struct toradex_eth_addr *eth_addr)
+{
+	int i;
+	u32 oui = ntohl(eth_addr->oui) >> 8;
+	u32 nic = ntohl(eth_addr->nic) >> 8;
+
+	for (i = 0; i < ARRAY_SIZE(toradex_ouis); i++) {
+		if (toradex_ouis[i] == oui)
+			break;
+	}
+
+	return (u32)((i << 24) + nic);
+}
+
+void get_mac_from_serial(u32 tdx_serial, struct toradex_eth_addr *eth_addr)
+{
+	u8 oui_index = tdx_serial >> 24;
+	u32 nic = tdx_serial & GENMASK(23, 0);
+	u32 oui;
+
+	if (oui_index >= ARRAY_SIZE(toradex_ouis)) {
+		puts("Can't find OUI for this serial#\n");
+		oui_index = 0;
+	}
+
+	oui = toradex_ouis[oui_index];
+
+	eth_addr->oui = htonl(oui << 8);
+	eth_addr->nic = htonl(nic << 8);
+}
+
 #ifdef CONFIG_TDX_CFG_BLOCK_IS_IN_MMC
 static int tdx_cfg_block_mmc_storage(u8 *config_block, int write)
 {
@@ -330,8 +361,7 @@ int read_tdx_cfg_block(void)
 				memcpy(&tdx_eth_addr, config_block + offset,
 				       6);
 
-				/* NIC part of MAC address is serial number */
-				tdx_serial = ntohl(tdx_eth_addr.nic) >> 8;
+				tdx_serial = get_serial_from_mac(&tdx_eth_addr);
 				break;
 			case TAG_HW:
 				memcpy(&tdx_hw_tag, config_block + offset, 8);
@@ -344,8 +374,7 @@ int read_tdx_cfg_block(void)
 	}
 
 	/* Cap product id to avoid issues with a yet unknown one */
-	if (tdx_hw_tag.prodid >= (sizeof(toradex_modules) /
-				  sizeof(toradex_modules[0])))
+	if (tdx_hw_tag.prodid >= ARRAY_SIZE(toradex_modules))
 		tdx_hw_tag.prodid = 0;
 
 out:
@@ -353,201 +382,63 @@ out:
 	return ret;
 }
 
+static int parse_assembly_string(char *string_to_parse, u16 *assembly)
+{
+	if (string_to_parse[3] >= 'A' && string_to_parse[3] <= 'Z')
+		*assembly = string_to_parse[3] - 'A';
+	else if (string_to_parse[3] == '#')
+		*assembly = dectoul(&string_to_parse[4], NULL);
+	else
+		return -EINVAL;
+
+	return 0;
+}
+
 static int get_cfgblock_interactive(void)
 {
 	char message[CONFIG_SYS_CBSIZE];
-	char *soc;
-	char it = 'n';
-	char wb = 'n';
 	int len = 0;
+	int ret = 0;
+	unsigned int prodid;
+	int i;
 
-	/* Unknown module by default */
-	tdx_hw_tag.prodid = 0;
+	printf("Enabled modules:\n");
+	for (i = 0; i < ARRAY_SIZE(toradex_modules); i++) {
+		if (toradex_modules[i].is_enabled)
+			printf(" %04d %s\n", i, toradex_modules[i].name);
+	}
 
-	if (cpu_is_pxa27x())
-		sprintf(message, "Is the module the 312 MHz version? [y/N] ");
-	else
-		it = 'y';
-
-#if defined(CONFIG_TARGET_APALIS_IMX8) || \
-		defined(CONFIG_TARGET_APALIS_IMX8X) || \
-		defined(CONFIG_TARGET_COLIBRI_IMX6ULL) || \
-		defined(CONFIG_TARGET_COLIBRI_IMX8X) || \
-		defined(CONFIG_TARGET_VERDIN_IMX8MM) || \
-		defined(CONFIG_TARGET_VERDIN_IMX8MP)
-	sprintf(message, "Does the module have Wi-Fi / Bluetooth? [y/N] ");
+	sprintf(message, "Enter the module ID: ");
 	len = cli_readline(message);
-	wb = console_buffer[0];
-#endif
 
-	soc = env_get("soc");
-	if (!strcmp("mx6", soc)) {
-#ifdef CONFIG_TARGET_APALIS_IMX6
-		if (it == 'y' || it == 'Y') {
-			if (is_cpu_type(MXC_CPU_MX6Q))
-				tdx_hw_tag.prodid = APALIS_IMX6Q_IT;
-			else
-				tdx_hw_tag.prodid = APALIS_IMX6D_IT;
-		} else {
-			if (is_cpu_type(MXC_CPU_MX6Q))
-				tdx_hw_tag.prodid = APALIS_IMX6Q;
-			else
-				tdx_hw_tag.prodid = APALIS_IMX6D;
-		}
-#elif CONFIG_TARGET_COLIBRI_IMX6
-		if (it == 'y' || it == 'Y') {
-			if (is_cpu_type(MXC_CPU_MX6DL))
-				tdx_hw_tag.prodid = COLIBRI_IMX6DL_IT;
-			else if (is_cpu_type(MXC_CPU_MX6SOLO))
-				tdx_hw_tag.prodid = COLIBRI_IMX6S_IT;
-		} else {
-			if (is_cpu_type(MXC_CPU_MX6DL))
-				tdx_hw_tag.prodid = COLIBRI_IMX6DL;
-			else if (is_cpu_type(MXC_CPU_MX6SOLO))
-				tdx_hw_tag.prodid = COLIBRI_IMX6S;
-		}
-#elif CONFIG_TARGET_COLIBRI_IMX6ULL
-		if (it == 'y' || it == 'Y') {
-			if (wb == 'y' || wb == 'Y')
-				tdx_hw_tag.prodid = COLIBRI_IMX6ULL_WIFI_BT_IT;
-			else
-				tdx_hw_tag.prodid = COLIBRI_IMX6ULL_IT;
-		} else {
-			if (wb == 'y' || wb == 'Y')
-				tdx_hw_tag.prodid = COLIBRI_IMX6ULL_WIFI_BT;
-			else
-				tdx_hw_tag.prodid = COLIBRI_IMX6ULL;
-		}
-#endif
-	} else if (!strcmp("imx7d", soc))
-		tdx_hw_tag.prodid = COLIBRI_IMX7D;
-	else if (!strcmp("imx7s", soc))
-		tdx_hw_tag.prodid = COLIBRI_IMX7S;
-	else if (is_cpu_type(MXC_CPU_IMX8QM)) {
-		if (it == 'y' || it == 'Y') {
-			if (wb == 'y' || wb == 'Y')
-				tdx_hw_tag.prodid = APALIS_IMX8QM_WIFI_BT_IT;
-			else
-				tdx_hw_tag.prodid = APALIS_IMX8QM_IT;
-		} else {
-			if (wb == 'y' || wb == 'Y')
-				tdx_hw_tag.prodid = APALIS_IMX8QP_WIFI_BT;
-			else
-				tdx_hw_tag.prodid = APALIS_IMX8QP;
-		}
-	} else if (is_cpu_type(MXC_CPU_IMX8QXP)) {
-#ifdef CONFIG_TARGET_APALIS_IMX8X
-		if (it == 'y' || it == 'Y' || wb == 'y' || wb == 'Y') {
-				tdx_hw_tag.prodid = APALIS_IMX8QXP_WIFI_BT_IT;
-		} else {
-			if (gd->ram_size == 0x40000000)
-				tdx_hw_tag.prodid = APALIS_IMX8DXP;
-			else
-				tdx_hw_tag.prodid = APALIS_IMX8QXP;
-		}
-#elif CONFIG_TARGET_COLIBRI_IMX8X
-		if (it == 'y' || it == 'Y') {
-			if (wb == 'y' || wb == 'Y')
-				tdx_hw_tag.prodid = COLIBRI_IMX8QXP_WIFI_BT_IT;
-			else
-				tdx_hw_tag.prodid = COLIBRI_IMX8QXP_IT;
-		} else {
-			if (wb == 'y' || wb == 'Y')
-				tdx_hw_tag.prodid = COLIBRI_IMX8DX_WIFI_BT;
-			else
-				tdx_hw_tag.prodid = COLIBRI_IMX8DX;
-		}
-#endif
-	} else if (is_cpu_type(MXC_CPU_IMX8MMDL)) {
-		if (wb == 'y' || wb == 'Y')
-			tdx_hw_tag.prodid = VERDIN_IMX8MMDL_WIFI_BT_IT;
-		else
-			tdx_hw_tag.prodid = VERDIN_IMX8MMDL;
-	} else if (is_cpu_type(MXC_CPU_IMX8MM)) {
-		if (wb == 'y' || wb == 'Y')
-			tdx_hw_tag.prodid = VERDIN_IMX8MMQ_WIFI_BT_IT;
-		else
-			tdx_hw_tag.prodid = VERDIN_IMX8MMQ_IT;
-	} else if (is_cpu_type(MXC_CPU_IMX8MN)) {
-		tdx_hw_tag.prodid = VERDIN_IMX8MNQ_WIFI_BT;
-	} else if (is_cpu_type(MXC_CPU_IMX8MP)) {
-		if (wb == 'y' || wb == 'Y')
-			tdx_hw_tag.prodid = VERDIN_IMX8MPQ_WIFI_BT_IT;
-		else
-			tdx_hw_tag.prodid = VERDIN_IMX8MPQ;
-	} else if (!strcmp("tegra20", soc)) {
-		if (it == 'y' || it == 'Y')
-			if (gd->ram_size == 0x10000000)
-				tdx_hw_tag.prodid = COLIBRI_T20_256MB_IT;
-			else
-				tdx_hw_tag.prodid = COLIBRI_T20_512MB_IT;
-		else
-			if (gd->ram_size == 0x10000000)
-				tdx_hw_tag.prodid = COLIBRI_T20_256MB;
-			else
-				tdx_hw_tag.prodid = COLIBRI_T20_512MB;
-	} else if (cpu_is_pxa27x()) {
-		if (it == 'y' || it == 'Y')
-			tdx_hw_tag.prodid = COLIBRI_PXA270_312MHZ;
-		else
-			tdx_hw_tag.prodid = COLIBRI_PXA270_520MHZ;
-	}
-#ifdef CONFIG_MACH_TYPE
-	else if (!strcmp("tegra30", soc)) {
-		if (CONFIG_MACH_TYPE == MACH_TYPE_APALIS_T30) {
-			if (it == 'y' || it == 'Y')
-				tdx_hw_tag.prodid = APALIS_T30_IT;
-			else
-				if (gd->ram_size == 0x40000000)
-					tdx_hw_tag.prodid = APALIS_T30_1GB;
-				else
-					tdx_hw_tag.prodid = APALIS_T30_2GB;
-		} else {
-			if (it == 'y' || it == 'Y')
-				tdx_hw_tag.prodid = COLIBRI_T30_IT;
-			else
-				tdx_hw_tag.prodid = COLIBRI_T30;
-		}
-	}
-#endif /* CONFIG_MACH_TYPE */
-	else if (!strcmp("tegra124", soc)) {
-		tdx_hw_tag.prodid = APALIS_TK1_2GB;
-	} else if (!strcmp("vf500", soc)) {
-		if (it == 'y' || it == 'Y')
-			tdx_hw_tag.prodid = COLIBRI_VF50_IT;
-		else
-			tdx_hw_tag.prodid = COLIBRI_VF50;
-	} else if (!strcmp("vf610", soc)) {
-		if (it == 'y' || it == 'Y')
-			tdx_hw_tag.prodid = COLIBRI_VF61_IT;
-		else
-			tdx_hw_tag.prodid = COLIBRI_VF61;
-	}
-
-	if (!tdx_hw_tag.prodid) {
-		printf("Module type not detectable due to unknown SoC\n");
+	prodid = dectoul(console_buffer, NULL);
+	if (prodid >= ARRAY_SIZE(toradex_modules) || !toradex_modules[prodid].is_enabled) {
+		printf("Parsing module id failed\n");
 		return -1;
 	}
+	tdx_hw_tag.prodid = prodid;
 
+	len = 0;
 	while (len < 4) {
-		sprintf(message, "Enter the module version (e.g. V1.1B): V");
+		sprintf(message, "Enter the module version (e.g. V1.1B or V1.1#26): V");
 		len = cli_readline(message);
 	}
 
 	tdx_hw_tag.ver_major = console_buffer[0] - '0';
 	tdx_hw_tag.ver_minor = console_buffer[2] - '0';
-	tdx_hw_tag.ver_assembly = console_buffer[3] - 'A';
 
-	if (cpu_is_pxa27x() && tdx_hw_tag.ver_major == 1)
-		tdx_hw_tag.prodid -= (COLIBRI_PXA270_312MHZ -
-				       COLIBRI_PXA270_V1_312MHZ);
+	ret = parse_assembly_string(console_buffer, &tdx_hw_tag.ver_assembly);
+	if (ret) {
+		printf("Parsing module version failed\n");
+		return ret;
+	}
 
 	while (len < 8) {
 		sprintf(message, "Enter module serial number: ");
 		len = cli_readline(message);
 	}
 
-	tdx_serial = simple_strtoul(console_buffer, NULL, 10);
+	tdx_serial = dectoul(console_buffer, NULL);
 
 	return 0;
 }
@@ -555,6 +446,8 @@ static int get_cfgblock_interactive(void)
 static int get_cfgblock_barcode(char *barcode, struct toradex_hw *tag,
 				u32 *serial)
 {
+	char revision[3] = {barcode[6], barcode[7], '\0'};
+
 	if (strlen(barcode) < 16) {
 		printf("Argument too short, barcode is 16 chars long\n");
 		return -1;
@@ -563,14 +456,14 @@ static int get_cfgblock_barcode(char *barcode, struct toradex_hw *tag,
 	/* Get hardware information from the first 8 digits */
 	tag->ver_major = barcode[4] - '0';
 	tag->ver_minor = barcode[5] - '0';
-	tag->ver_assembly = barcode[7] - '0';
+	tag->ver_assembly = dectoul(revision, NULL);
 
 	barcode[4] = '\0';
-	tag->prodid = simple_strtoul(barcode, NULL, 10);
+	tag->prodid = dectoul(barcode, NULL);
 
 	/* Parse second part of the barcode (serial number */
 	barcode += 8;
-	*serial = simple_strtoul(barcode, NULL, 10);
+	*serial = dectoul(barcode, NULL);
 
 	return 0;
 }
@@ -707,7 +600,7 @@ int try_migrate_tdx_cfg_block_carrier(void)
 	tdx_car_hw_tag.ver_assembly = pid8[7] - '0';
 
 	pid8[4] = '\0';
-	tdx_car_hw_tag.prodid = simple_strtoul(pid8, NULL, 10);
+	tdx_car_hw_tag.prodid = dectoul(pid8, NULL);
 
 	/* Valid Tag */
 	write_tag(config_block, &offset, TAG_VALID, NULL, 0);
@@ -741,33 +634,38 @@ static int get_cfgblock_carrier_interactive(void)
 {
 	char message[CONFIG_SYS_CBSIZE];
 	int len;
+	int ret = 0;
 
 	printf("Supported carrier boards:\n");
 	printf("CARRIER BOARD NAME\t\t [ID]\n");
-	for (int i = 0; i < sizeof(toradex_carrier_boards) /
-			    sizeof(toradex_carrier_boards[0]); i++)
+	for (int i = 0; i < ARRAY_SIZE(toradex_carrier_boards); i++)
 		if (toradex_carrier_boards[i])
 			printf("%s \t\t [%d]\n", toradex_carrier_boards[i], i);
 
 	sprintf(message, "Choose your carrier board (provide ID): ");
 	len = cli_readline(message);
-	tdx_car_hw_tag.prodid = simple_strtoul(console_buffer, NULL, 10);
+	tdx_car_hw_tag.prodid = dectoul(console_buffer, NULL);
 
 	do {
-		sprintf(message, "Enter carrier board version (e.g. V1.1B): V");
+		sprintf(message, "Enter carrier board version (e.g. V1.1B or V1.1#26): V");
 		len = cli_readline(message);
 	} while (len < 4);
 
 	tdx_car_hw_tag.ver_major = console_buffer[0] - '0';
 	tdx_car_hw_tag.ver_minor = console_buffer[2] - '0';
-	tdx_car_hw_tag.ver_assembly = console_buffer[3] - 'A';
+
+	ret = parse_assembly_string(console_buffer, &tdx_car_hw_tag.ver_assembly);
+	if (ret) {
+		printf("Parsing module version failed\n");
+		return ret;
+	}
 
 	while (len < 8) {
 		sprintf(message, "Enter carrier board serial number: ");
 		len = cli_readline(message);
 	}
 
-	tdx_car_serial = simple_strtoul(console_buffer, NULL, 10);
+	tdx_car_serial = dectoul(console_buffer, NULL);
 
 	return 0;
 }
@@ -937,8 +835,7 @@ static int do_cfgblock_create(struct cmd_tbl *cmdtp, int flag, int argc,
 	}
 
 	/* Convert serial number to MAC address (the storage format) */
-	tdx_eth_addr.oui = htonl(0x00142dUL << 8);
-	tdx_eth_addr.nic = htonl(tdx_serial << 8);
+	get_mac_from_serial(tdx_serial, &tdx_eth_addr);
 
 	/* Valid Tag */
 	write_tag(config_block, &offset, TAG_VALID, NULL, 0);
