@@ -14,10 +14,6 @@
 #include <reset.h>
 #include "ohci.h"
 
-#if !defined(CONFIG_USB_OHCI_NEW)
-# error "Generic OHCI driver requires CONFIG_USB_OHCI_NEW"
-#endif
-
 struct generic_ohci {
 	ohci_t ohci;
 	struct clk *clocks;	/* clock list */
@@ -198,6 +194,6 @@ U_BOOT_DRIVER(ohci_generic) = {
 	.probe = ohci_usb_probe,
 	.remove = ohci_usb_remove,
 	.ops	= &ohci_usb_ops,
-	.priv_auto_alloc_size = sizeof(struct generic_ohci),
+	.priv_auto	= sizeof(struct generic_ohci),
 	.flags	= DM_FLAG_ALLOC_PRIV_DMA,
 };
