@@ -17,10 +17,6 @@
 /* SPL */
 
 #define CONFIG_SYS_UBOOT_START		CONFIG_SYS_TEXT_BASE
-#define CONFIG_SPL_BSS_START_ADDR	0x80010000
-#define CONFIG_SPL_BSS_MAX_SIZE		0x10000
-#define CONFIG_SPL_MAX_SIZE		0x10000
-#define CONFIG_SPL_PAD_TO		0
 
 /* Dummy value */
 #define CONFIG_SYS_UBOOT_BASE		0
@@ -39,11 +35,6 @@
 
 /* RAM */
 
-/* Memory usage */
-#define CONFIG_SYS_MAXARGS		64
-#define CONFIG_SYS_BOOTPARAMS_LEN	(128 * 1024)
-#define CONFIG_SYS_CBSIZE		512
-
 /* Environment settings */
 
 #endif /* __CONFIG_GARDENA_SMART_GATEWAY_H */
