@@ -9,18 +9,14 @@
 
 #include "ls1046a_common.h"
 
-#define CONFIG_LAYERSCAPE_NS_ACCESS
-
 /* Physical Memory Map */
 
 #define SPD_EEPROM_ADDRESS		0x51
-#define CONFIG_SYS_SPD_BUS_NUM		0
 
 #define CONFIG_MEM_INIT_VALUE           0xdeadbeef
 
 #if defined(CONFIG_QSPI_BOOT)
 #define CONFIG_SYS_UBOOT_BASE		0x40100000
-#define CONFIG_SYS_SPL_ARGS_ADDR	0x90000000
 #endif
 
 #define CONFIG_SYS_NAND_BASE		0x7e800000
@@ -139,8 +135,6 @@
 #endif
 #endif
 
-#define CONFIG_SPL_FS_LOAD_PAYLOAD_NAME "u-boot.img"
-
 #include <asm/fsl_secure_boot.h>
 
 #endif /* __LS1046ARDB_H__ */
