@@ -8,15 +8,17 @@ import shutil
 import subprocess
 import sys
 
-from buildman import board
+from buildman import boards
 from buildman import bsettings
+from buildman import cfgutil
 from buildman import toolchain
 from buildman.builder import Builder
 from patman import command
 from patman import gitutil
 from patman import patchstream
 from patman import terminal
-from patman.terminal import Print
+from patman import tools
+from patman.terminal import tprint
 
 def GetPlural(count):
     """Returns a plural 's' if count is not 1"""
@@ -71,7 +73,7 @@ def ShowActions(series, why_selected, boards_selected, builder, options,
     if commits:
         for upto in range(0, len(series.commits), options.step):
             commit = series.commits[upto]
-            print('   ', col.Color(col.YELLOW, commit.hash[:8], bright=False), end=' ')
+            print('   ', col.build(col.YELLOW, commit.hash[:8], bright=False), end=' ')
             print(commit.subject)
     print()
     for arg in why_selected:
@@ -83,9 +85,9 @@ def ShowActions(series, why_selected, boards_selected, builder, options,
             len(why_selected['all'])))
     if board_warnings:
         for warning in board_warnings:
-            print(col.Color(col.YELLOW, warning))
+            print(col.build(col.YELLOW, warning))
 
-def ShowToolchainPrefix(boards, toolchains):
+def ShowToolchainPrefix(brds, toolchains):
     """Show information about a the tool chain used by one or more boards
 
     The function checks that all boards use the same toolchain, then prints
@@ -98,9 +100,9 @@ def ShowToolchainPrefix(boards, toolchains):
     Return:
         None on success, string error message otherwise
     """
-    boards = boards.GetSelectedDict()
+    board_selected = brds.get_selected_dict()
     tc_set = set()
-    for brd in boards.values():
+    for brd in board_selected.values():
         tc_set.add(toolchains.Select(brd.arch))
     if len(tc_set) != 1:
         return 'Supplied boards must share one toolchain'
@@ -109,8 +111,8 @@ def ShowToolchainPrefix(boards, toolchains):
     print(tc.GetEnvArgs(toolchain.VAR_CROSS_COMPILE))
     return None
 
-def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
-               clean_dir=False):
+def DoBuildman(options, args, toolchains=None, make_func=None, brds=None,
+               clean_dir=False, test_thread_exceptions=False):
     """The main control code for buildman
 
     Args:
@@ -122,21 +124,23 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
                 to execute 'make'. If this is None, the normal function
                 will be used, which calls the 'make' tool with suitable
                 arguments. This setting is useful for tests.
-        board: Boards() object to use, containing a list of available
+        brds: Boards() object to use, containing a list of available
                 boards. If this is None it will be created and scanned.
+        clean_dir: Used for tests only, indicates that the existing output_dir
+            should be removed before starting the build
+        test_thread_exceptions: Uses for tests only, True to make the threads
+            raise an exception instead of reporting their result. This simulates
+            a failure in the code somewhere
     """
     global builder
 
     if options.full_help:
-        pager = os.getenv('PAGER')
-        if not pager:
-            pager = 'more'
-        fname = os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])),
-                             'README')
-        command.Run(pager, fname)
+        tools.print_full_help(
+            os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), 'README')
+        )
         return 0
 
-    gitutil.Setup()
+    gitutil.setup()
     col = terminal.Color()
 
     options.git_dir = os.path.join(options.git, '.git')
@@ -148,14 +152,14 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
     if options.fetch_arch:
         if options.fetch_arch == 'list':
             sorted_list = toolchains.ListArchs()
-            print(col.Color(col.BLUE, 'Available architectures: %s\n' %
+            print(col.build(col.BLUE, 'Available architectures: %s\n' %
                             ' '.join(sorted_list)))
             return 0
         else:
             fetch_arch = options.fetch_arch
             if fetch_arch == 'all':
                 fetch_arch = ','.join(toolchains.ListArchs())
-                print(col.Color(col.CYAN, '\nDownloading toolchains: %s' %
+                print(col.build(col.CYAN, '\nDownloading toolchains: %s' %
                                 fetch_arch))
             for arch in fetch_arch.split(','):
                 print()
@@ -172,32 +176,25 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
         print()
         return 0
 
-    if options.incremental:
-        print(col.Color(col.RED,
-                        'Warning: -I has been removed. See documentation'))
     if not options.output_dir:
         if options.work_in_output:
-            sys.exit(col.Color(col.RED, '-w requires that you specify -o'))
+            sys.exit(col.build(col.RED, '-w requires that you specify -o'))
         options.output_dir = '..'
 
     # Work out what subset of the boards we are building
-    if not boards:
+    if not brds:
         if not os.path.exists(options.output_dir):
             os.makedirs(options.output_dir)
         board_file = os.path.join(options.output_dir, 'boards.cfg')
-        our_path = os.path.dirname(os.path.realpath(__file__))
-        genboardscfg = os.path.join(our_path, '../genboardscfg.py')
-        if not os.path.exists(genboardscfg):
-            genboardscfg = os.path.join(options.git, 'tools/genboardscfg.py')
-        status = subprocess.call([genboardscfg, '-q', '-o', board_file])
-        if status != 0:
-            # Older versions don't support -q
-            status = subprocess.call([genboardscfg, '-o', board_file])
-            if status != 0:
-                sys.exit("Failed to generate boards.cfg")
 
-        boards = board.Boards()
-        boards.ReadBoards(board_file)
+        brds = boards.Boards()
+        ok = brds.ensure_board_list(board_file,
+                                    options.threads or multiprocessing.cpu_count(),
+                                    force=options.regen_board_list,
+                                    quiet=not options.verbose)
+        if options.regen_board_list:
+            return 0 if ok else 2
+        brds.read_boards(board_file)
 
     exclude = []
     if options.exclude:
@@ -210,16 +207,16 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
             requested_boards += b.split(',')
     else:
         requested_boards = None
-    why_selected, board_warnings = boards.SelectBoards(args, exclude,
-                                                       requested_boards)
-    selected = boards.GetSelected()
+    why_selected, board_warnings = brds.select_boards(args, exclude,
+                                                      requested_boards)
+    selected = brds.get_selected()
     if not len(selected):
-        sys.exit(col.Color(col.RED, 'No matching boards found'))
+        sys.exit(col.build(col.RED, 'No matching boards found'))
 
     if options.print_prefix:
-        err = ShowToolchainPrefix(boards, toolchains)
+        err = ShowToolchainPrefix(brds, toolchains)
         if err:
-            sys.exit(col.Color(col.RED, err))
+            sys.exit(col.build(col.RED, err))
         return 0
 
     # Work out how many commits to build. We want to build everything on the
@@ -232,30 +229,30 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
             count = 1
         else:
             if has_range:
-                count, msg = gitutil.CountCommitsInRange(options.git_dir,
+                count, msg = gitutil.count_commits_in_range(options.git_dir,
                                                          options.branch)
             else:
-                count, msg = gitutil.CountCommitsInBranch(options.git_dir,
+                count, msg = gitutil.count_commits_in_branch(options.git_dir,
                                                           options.branch)
             if count is None:
-                sys.exit(col.Color(col.RED, msg))
+                sys.exit(col.build(col.RED, msg))
             elif count == 0:
-                sys.exit(col.Color(col.RED, "Range '%s' has no commits" %
+                sys.exit(col.build(col.RED, "Range '%s' has no commits" %
                                    options.branch))
             if msg:
-                print(col.Color(col.YELLOW, msg))
+                print(col.build(col.YELLOW, msg))
             count += 1   # Build upstream commit also
 
     if not count:
         str = ("No commits found to process in branch '%s': "
                "set branch's upstream or use -c flag" % options.branch)
-        sys.exit(col.Color(col.RED, str))
+        sys.exit(col.build(col.RED, str))
     if options.work_in_output:
         if len(selected) != 1:
-            sys.exit(col.Color(col.RED,
+            sys.exit(col.build(col.RED,
                                '-w can only be used with a single board'))
         if count != 1:
-            sys.exit(col.Color(col.RED,
+            sys.exit(col.build(col.RED,
                                '-w can only be used with a single commit'))
 
     # Read the metadata from the commits. First look at the upstream commit,
@@ -272,9 +269,9 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
             if has_range:
                 range_expr = options.branch
             else:
-                range_expr = gitutil.GetRangeInBranch(options.git_dir,
+                range_expr = gitutil.get_range_in_branch(options.git_dir,
                                                       options.branch)
-            upstream_commit = gitutil.GetUpstream(options.git_dir,
+            upstream_commit = gitutil.get_upstream(options.git_dir,
                                                   options.branch)
             series = patchstream.get_metadata_for_list(upstream_commit,
                 options.git_dir, 1, series=None, allow_overwrite=True)
@@ -294,7 +291,7 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
 
     # By default we have one thread per CPU. But if there are not enough jobs
     # we can have fewer threads and use a high '-j' value for make.
-    if not options.threads:
+    if options.threads is None:
         options.threads = min(multiprocessing.cpu_count(), len(selected))
     if not options.jobs:
         options.jobs = max(1, (multiprocessing.cpu_count() +
@@ -303,7 +300,7 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
     if not options.step:
         options.step = len(series.commits) - 1
 
-    gnu_make = command.Output(os.path.join(options.git,
+    gnu_make = command.output(os.path.join(options.git,
             'scripts/show-gnu-make'), raise_on_error=False).rstrip()
     if not gnu_make:
         sys.exit('GNU Make not found')
@@ -318,6 +315,8 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
             output_dir = os.path.join(options.output_dir, dirname)
         if clean_dir and os.path.exists(output_dir):
             shutil.rmtree(output_dir)
+    adjust_cfg = cfgutil.convert_list_to_dict(options.adjust_cfg)
+
     builder = Builder(toolchains, output_dir, options.git_dir,
             options.threads, options.jobs, gnu_make=gnu_make, checkout=True,
             show_unknown=options.show_unknown, step=options.step,
@@ -328,7 +327,9 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
             config_only=options.config_only,
             squash_config_y=not options.preserve_config_y,
             warnings_as_errors=options.warnings_as_errors,
-            work_in_output=options.work_in_output)
+            work_in_output=options.work_in_output,
+            test_thread_exceptions=test_thread_exceptions,
+            adjust_cfg=adjust_cfg)
     builder.force_config_on_failure = not options.quick
     if make_func:
         builder.do_make = make_func
@@ -344,7 +345,7 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
         builder.in_tree = options.in_tree
 
         # Work out which boards to build
-        board_selected = boards.GetSelectedDict()
+        board_selected = brds.get_selected_dict()
 
         if series:
             commits = series.commits
@@ -354,8 +355,9 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
         else:
             commits = None
 
-        Print(GetActionSummary(options.summary, commits, board_selected,
-                               options))
+        if not options.ide:
+            tprint(GetActionSummary(options.summary, commits, board_selected,
+                                    options))
 
         # We can't show function sizes without board details at present
         if options.show_bloat:
@@ -364,13 +366,15 @@ def DoBuildman(options, args, toolchains=None, make_func=None, boards=None,
             options.show_errors, options.show_sizes, options.show_detail,
             options.show_bloat, options.list_error_boards, options.show_config,
             options.show_environment, options.filter_dtb_warnings,
-            options.filter_migration_warnings)
+            options.filter_migration_warnings, options.ide)
         if options.summary:
             builder.ShowSummary(commits, board_selected)
         else:
-            fail, warned = builder.BuildBoards(commits, board_selected,
-                                options.keep_outputs, options.verbose)
-            if fail:
+            fail, warned, excs = builder.BuildBoards(
+                commits, board_selected, options.keep_outputs, options.verbose)
+            if excs:
+                return 102
+            elif fail:
                 return 100
             elif warned and not options.ignore_warnings:
                 return 101
